@@ -18,7 +18,7 @@
 #' @param L Lower bound for the generalized returns to scale (grs).
 #' @param U Upper bound for the generalized returns to scale (grs).
 #' @param B Number of bootstrap iterations.
-#' @param h Bandwith or smoothing window. By default h=0.014 (You can set h to any other value). The optimal bandwidth factor can also be calculated following the proposals of Silverman (1986) and Dario y Simar (2007). So, h="h1" is the optimal h referred as "robust normal-reference rule" (Dario and Simar, 2007 p.60), h="h2" is the value of h1 but instead of the factor 1.06 with the factor 0.9, and h="h3" is the value of h1 adjusted for scale and sample size (Dario and Simar, 2007 p.61). 
+#' @param h Bandwith or smoothing window. By default h=0.014 (You can set h to any other value). The optimal bandwidth factor can also be calculated following the proposals of Silverman (1986) and Dario y Simar (2007). So, h="h1" is the optimal h referred as "robust normal-reference rule" (Dario and Simar, 2007 p.60), h="h2" is the value of h1 but instead of the factor 1.06 with the factor 0.9, h="h3" is the value of h1 adjusted for scale and sample size (Dario and Simar, 2007 p.61), and h="h4" is the bandwith provided by a Gaussian kernel destiy estimate. 
 #' @param alpha Between 0 and 1 (for confidence intervals).
 #'   
 #' @author 
@@ -33,7 +33,12 @@
 #'
 #' University of Valencia (Spain) 
 #' 
-#' @references  Daraio, C.; Simar, L. (2007). Advanced Robust and Nonparametric Methods in Efficiency Analysis: Methodology and Applications. New York: Springer.
+#' @references  
+#' Behr, A. (2015). Production and Efficiency Analysis with R. Springer.
+#' 
+#' Bogetoft, P.; Otto, L. (2010).  Benchmarking with DEA, SFA, and R. Springer.
+#' 
+#' Daraio, C.; Simar, L. (2007). Advanced Robust and Nonparametric Methods in Efficiency Analysis: Methodology and Applications. New York: Springer.
 #' 
 #' Färe, R.; Grosskopf, S.; Kokkenlenberg, E. (1989). "Measuring Plant Capacity, Utilization and Technical Change: A Nonparametric Approach". International Economic Review, 30(3), 655-666. 
 #' 
@@ -62,7 +67,7 @@
 #' result$CI
 #' 
 #' @import lpSolve 
-#' @importFrom stats IQR median quantile rnorm runif sd var
+#' @importFrom stats IQR median quantile rnorm runif sd var density
 #' 
 #' @export
   
@@ -79,7 +84,6 @@ bootstrap_basic <- function(datadea,
   if (!is.deadata(datadea)) {
     stop("Data should be of class deadata. Run read_data function first!")
   }
-  
   if (!is.null(datadea$ud_inputs) || !is.null(datadea$ud_outputs)) {
     warning("This model does not take into account the undesirable feature for inputs/outputs.")
   }
@@ -89,15 +93,10 @@ bootstrap_basic <- function(datadea,
   if (!is.null(datadea$nd_inputs) || !is.null(datadea$nd_outputs)) {
     warning("This model does not take into account the non-discretionary feature for inputs/outputs.")
   }
-  
-  # Checking orientation
   orientation <- tolower(orientation)
   orientation <- match.arg(orientation)
-  
-  # Checking rts
   rts <- tolower(rts)
   rts <- match.arg(rts)
-  
   if (rts == "grs") {
     if (L > 1) {
       stop("L must be <= 1.")
@@ -106,10 +105,8 @@ bootstrap_basic <- function(datadea,
       stop("U must be >= 1.")
     }
   }
-  
   dmunames <- datadea$dmunames
-  nd <- length(dmunames) # number of dmus
-  
+  nd <- length(dmunames)
   if (orientation == "io") {
     input <- datadea$input
     output <- datadea$output
@@ -119,19 +116,15 @@ bootstrap_basic <- function(datadea,
     output <- -datadea$input
     obj <- "max"
   }
+  
   inputnames <- rownames(input)
   outputnames <- rownames(output)
-  ni <- nrow(input) # number of  inputs
-  no <- nrow(output) # number of outputs
-  
+  ni <- nrow(input)
+  no <- nrow(output)
   score <- rep(0, nd)
   names(score) <- dmunames
   score_bc <- score
-  
-  ###########################
-  
   f.obj <- c(1, rep(0, nd))
-  
   if (rts == "crs") {
     f.con.rs <- NULL
     f.con2.rs <- NULL
@@ -139,169 +132,143 @@ bootstrap_basic <- function(datadea,
     f.rhs.rs <- NULL
   } else {
     f.con.rs <- cbind(0, matrix(1, nrow = 1, ncol = nd))
-    f.con2.rs <- cbind(matrix(1, nrow = 1, ncol = nd), matrix(0, nrow = 1, ncol = ni + no))
+    f.con2.rs <- cbind(matrix(1, nrow = 1, ncol = nd), matrix(0, 
+                                                              nrow = 1, ncol = ni + no))
     f.rhs.rs <- 1
     if (rts == "vrs") {
       f.dir.rs <- "="
-    } else if (rts == "nirs") {
+    }     else if (rts == "nirs") {
       f.dir.rs <- "<="
-    } else if (rts == "ndrs") {
+    }    else if (rts == "ndrs") {
       f.dir.rs <- ">="
-    } else {
+    }    else {
       f.con.rs <- rbind(f.con.rs, f.con.rs)
       f.con2.rs <- rbind(f.con2.rs, f.con2.rs)
       f.dir.rs <- c(">=", "<=")
       f.rhs.rs <- c(L, U)
     }
   }
-  
   f.dir <- c(rep("<=", ni), rep(">=", no), f.dir.rs)
-  
   f.con.2 <- cbind(matrix(0, nrow = no, ncol = 1), output)
-  
   for (i in 1:nd) {
-    
     f.con.1 <- cbind(-input[, i], input)
     f.con <- rbind(f.con.1, f.con.2, f.con.rs)
-    
     f.rhs <- c(rep(0, ni), output[, i], f.rhs.rs)
-    
     score[i] <- lp(obj, f.obj, f.con, f.dir, f.rhs)$solution[1]
-    
+  }
+  if (orientation == "io") {
+    score_sp <- 1/score
+  } else {
+    score_sp <-  score
   }
   
-  if(orientation == "io") {
-    score_sp <- 1 / score
-  } else {
-    score_sp <- score
-  }
+  new_set <- round(c(score_sp, 2 - score_sp),6)
   
   if (is.null(h)) {
     
     h <- 0.014
     
-  } else if (!is.numeric(h)) {
-    
-    if (h %in% c("h1", "h2", "h3")) {
-      
-      score_h <- score[score_sp > 1.000001]
-      
-      if(length(score_h) == nd) { # Daraio and Simar (2007, p.60)
-        stop("It is not possible to calculate h. The number of inefficient DMUs must be less than the total number of DMUs.")
-      }
-      
-      new_set <- c(2 - score_h, score_h) 
-      
+  }   else if (!is.numeric(h)) {
+    if (h %in% c("h1", "h2", "h3", "h4")) {
       sd_new_set <- sd(new_set)
       iqr_new_set <- IQR(new_set)
-      
-      # Silverman (1986, equation 3.30, p.47)
-      if(sd_new_set > iqr_new_set / 1.34) {
-        desviation <- iqr_new_set / 1.34
-      } else{
+      if (sd_new_set > iqr_new_set/1.34) {
+        desviation <- iqr_new_set/1.34
+      }       else {
         desviation <- sd_new_set
       }
       
-      # Implementado eq. 3.29 silverman (en Daraio y simar es 1.06)
       if (h == "h1") {
-        h <- 1.06 * desviation * length(new_set) ^ (-1/5)
-      } else if (h == "h2") {
-        h <- 0.9 * desviation * length(new_set) ^ (-1/5)
-      } else { 
-        # ?ajustar hopt (eq. 3.26 de Daraio y Simar (2007) y Simar y Wilson 2006a?
-        h <- (1.06 * desviation * length(new_set) ^ (-1/5)) * (length(new_set) / nd)* (sd(score_sp) / sd(new_set))
-      } 
+        h <- 1.06 * desviation * length(new_set)^(-1/5)
+      }
+      else if (h == "h2") {
+        h <- 0.9 * desviation * length(new_set)^(-1/5)
+      }
+      else if (h == "h3") {
+        #ajustar hopt (eq. 3.26 de Daraio y Simar (2007) y Simar y Wilson 2006a como en Bogetoft and Otto (2010)
+        h <- (0.9 * desviation * length(new_set)^(-1/5)) * 
+          (length(new_set)/nd)^(1/5) * (sd(score_sp)/sd(new_set))
+      }
+      else {
+        # bandwith provided by a Gaussian kernel density estimate
+        dens <-  density(new_set)
+        h <- dens$bw 
+      }
       
-    } else {
+    }
+    else {
       stop("Incorrect bandwidth argument h.")
     }
-    
-  } 
-
-  # algoritmo de Simar y Wilson (1998, p.56-57)
-  estimates_bootstrap <- matrix(nrow = B, ncol = nd) # EFICIENCIA DE LAS DMUS PARA CADA REPETICION bootstrapp
+  }
+  
+  estimates_bootstrap <- matrix(nrow = B, ncol = nd)
   colnames(estimates_bootstrap) <- dmunames
-  
-  sigma_score <- var(score)
-  
-  thetatilde <- rep(0, nd)
-  
-  for (b in 1:B) {
-    
-    beta <- sample(score, nd, replace = TRUE)   
+  sigma_score <- var(new_set)
 
+  for (b in 1:B) {
+    beta <- sample(new_set, nd, replace = TRUE)
     epsilon <- rnorm(nd)
-  
-    if (orientation == "io") { 
-      for (i in 1:nd) {  
-        if (beta[i] + h * epsilon[i] <= 1) {
-          thetatilde[i] <- beta[i] + h * epsilon[i] # hopt calculated following Silverman
-        } 
-        else {
-          thetatilde[i] <- 2 - beta[i] - h * epsilon[i]
-        }
-      }
-    } 
-    if (orientation ==  "oo") {
-        for (i in 1:nd) {  
-          if (beta[i] + h * epsilon[i] >= 1) {
-            thetatilde[i] <- beta[i] + h * epsilon[i] # hopt calculated following Silverman
-          } 
-          else {
-            thetatilde[i] <- 2 - (beta[i] + h * epsilon[i])
-          }
-        }
+    thetatilde <- beta + h * epsilon
+    
+    thetatilde_est = 1 + 1/sqrt(1+h^2/sigma_score)*(thetatilde-1)
+    
+    thetatilde_est <- ifelse(thetatilde_est >= 1,
+                             thetatilde_est,
+                             2 - thetatilde_est)
+    
+    if(orientation=="io"){
+      input_corrected <- t((thetatilde_est/score_sp) * t(input))  #cambia en funcion de la orientacion
+    } else{
+      input_corrected <- t((score_sp/thetatilde_est) * t(input))  #cambia en funcion de la orientacion
     }
     
-    seq_bootstrap = mean(beta) + (thetatilde - mean(beta)) / sqrt(1 + h ^ 2 / sigma_score)
-    
-    input_corrected <- t((score / seq_bootstrap) * t(input)) 
-    
     for (i in 1:nd) {
-      
       f.con.1 <- cbind(-input[, i], input_corrected)
       f.con <- rbind(f.con.1, f.con.2, f.con.rs)
-      
       f.rhs <- c(rep(0, ni), output[, i], f.rhs.rs)
-      
-      estimates_bootstrap[b, i] <- lp(obj, f.obj, f.con, f.dir, f.rhs)$solution[1]
-      
+      estimates_bootstrap[b, i] <- lp(obj, f.obj, f.con, 
+                                      f.dir, f.rhs)$solution[1]
     }
     
-  } 
+  }
   
-  mean_estimates_boot <- apply(estimates_bootstrap, 2, mean)
-  var_estimates_boot <- apply(estimates_bootstrap, 2, var)
+  if (orientation == "io") {
+    estimates_bootstrap <- 1/estimates_bootstrap
+  }
   
-  bias <- colMeans(estimates_bootstrap) - score
+  mean_estimates_boot <- apply(estimates_bootstrap, 2, mean, na.rm=TRUE)
+  var_estimates_boot <- apply(estimates_bootstrap, 2, var, na.rm=TRUE)
+  median_estimates_boot <- apply(estimates_bootstrap, 2, median, na.rm=TRUE)
+  bias <- mean_estimates_boot - score_sp
+  score_bc <- score_sp - bias
+  
+  # Intervalos de confianza
+  estimates_boot_corrected <- t(apply(-estimates_bootstrap, 1, function(x) x + 2*score_sp))
+  CI_low <- apply(estimates_boot_corrected, 2, quantile, alpha/2, na.rm=TRUE)
+  CI_up <- apply(estimates_boot_corrected, 2, quantile, 1 - alpha/2, na.rm=TRUE)
+  CI <- data.frame(CI_low, CI_up)
+  
+  # Resultados en input oriented
+  if (orientation == "io") {
+    score <- 1/score_sp
+    estimates_bootstrap <- 1/estimates_bootstrap
+    score_bc <- 1/score_bc
+    CI <- t(apply(1/CI,1,rev))
+    colnames(CI) <- c("CI_low","CI_up")
+  }
   
-  score_bc <- score - bias
   names(score_bc) <- dmunames
-  
-  median_estimates_boot <- apply(estimates_bootstrap, 2, median) - 2 * bias
   names(mean_estimates_boot) <- dmunames
   names(var_estimates_boot) <- dmunames
   names(median_estimates_boot) <- dmunames
-  descriptives <- data.frame(mean_estimates_boot, var_estimates_boot, median_estimates_boot)
-  
-  CI_low <- apply(estimates_bootstrap, 2, quantile, probs = alpha / 2) -  2 * bias
-  CI_up <- apply(estimates_bootstrap, 2, quantile, probs = 1 - alpha / 2) - 2 * bias
   names(CI_low) <- dmunames
   names(CI_up) <- dmunames
+  descriptives <- data.frame(mean_estimates_boot, var_estimates_boot, 
+                             median_estimates_boot)
   
-  CI <- data.frame(CI_low, CI_up)
-  
-  res <- list(modelname = "bootstrap",
-              orientation = orientation,
-              rts = rts,
-              L = L,
-              U = U,
-              score = score,
-              score_bc = score_bc,
-              descriptives = descriptives,
-              CI = CI,
-              estimates_bootstrap = estimates_bootstrap,
+  res <- list(modelname = "bootstrap", orientation = orientation, 
+              rts = rts, L = L, U = U, score = score, bandwith = h, score_bc = score_bc, bias = bias,
+              descriptives = descriptives, CI = CI, estimates_bootstrap = estimates_bootstrap, 
               data = datadea)
-         
   return(structure(res, class = "dea"))
 }
